package prngAES;

import javacard.framework.APDU;
import javacard.framework.Applet;
import javacard.framework.ISO7816;
import javacard.framework.ISOException;
import javacard.framework.JCSystem;
import javacard.framework.Util;
import javacardx.crypto.*;
import javacard.security.*;

public class PrngAES extends Applet {

	//==================== Cyclic Seed File ========================================
	private static byte[] CyclicSeedFile = { (byte) 0x37, (byte) 0x7a,
		(byte) 0xbc, (byte) 0xc0, (byte) 0xea, (byte) 0x07, (byte) 0x94,
		(byte) 0x59, (byte) 0xd6, (byte) 0x37, (byte) 0x6b, (byte) 0x4c,
		(byte) 0x82, (byte) 0xdb, (byte) 0x54, (byte) 0xb2, (byte) 0xe8,
		(byte) 0xea, (byte) 0x71, (byte) 0xe1, (byte) 0xa4, (byte) 0x41,
		(byte) 0x06, (byte) 0x44, (byte) 0xfe, (byte) 0x86, (byte) 0x8e,
		(byte) 0x4f, (byte) 0x39, (byte) 0xf5, (byte) 0xdb, (byte) 0xd1,
		(byte) 0xf1, (byte) 0xc5, (byte) 0xd8, (byte) 0xac, (byte) 0xbb,
		(byte) 0x73, (byte) 0x51, (byte) 0xa1, (byte) 0xa3, (byte) 0x8a,
		(byte) 0x26, (byte) 0x5d, (byte) 0xf3, (byte) 0x61, (byte) 0x55,
		(byte) 0x56, (byte) 0x39, (byte) 0x3f, (byte) 0x4c, (byte) 0x2a,
		(byte) 0x43, (byte) 0xc4, (byte) 0xd7, (byte) 0xa1, (byte) 0xaa,
		(byte) 0xc1, (byte) 0xf2, (byte) 0xd6, (byte) 0x07, (byte) 0xa8,
		(byte) 0x58, (byte) 0x9a, (byte) 0x70, (byte) 0x84, (byte) 0x15,
		(byte) 0x19, (byte) 0x56, (byte) 0x61, (byte) 0x3d, (byte) 0x88,
		(byte) 0x2a, (byte) 0x44, (byte) 0x54, (byte) 0x29, (byte) 0x29,
		(byte) 0x26, (byte) 0x36, (byte) 0x06, (byte) 0xfe, (byte) 0xad,
		(byte) 0x27, (byte) 0x13, (byte) 0x86, (byte) 0x0e, (byte) 0x85,
		(byte) 0x3c, (byte) 0x32, (byte) 0xe2, (byte) 0x38, (byte) 0xd2,
		(byte) 0x91, (byte) 0x82, (byte) 0x89, (byte) 0xce, (byte) 0x79,
		(byte) 0x02, (byte) 0x43, (byte) 0xfd, (byte) 0xaf, (byte) 0x18,
		(byte) 0xe8, (byte) 0x5b, (byte) 0xd4, (byte) 0x72, (byte) 0x03,
		(byte) 0x63, (byte) 0x2b, (byte) 0x29, (byte) 0x72, (byte) 0xe0,
		(byte) 0x92, (byte) 0x54, (byte) 0x06, (byte) 0x1c, (byte) 0x7f,
		(byte) 0xc7, (byte) 0x37, (byte) 0x93, (byte) 0x2f, (byte) 0x7a,
		(byte) 0x84, (byte) 0x95, (byte) 0xec, (byte) 0x5e, (byte) 0xa5,
		(byte) 0xf6, (byte) 0x4e, (byte) 0x7e, (byte) 0x1f, (byte) 0xe6,
		(byte) 0xe2, (byte) 0x04, (byte) 0x2e, (byte) 0x25, (byte) 0x7f,
		(byte) 0x2f, (byte) 0x3c, (byte) 0xfe, (byte) 0x57, (byte) 0x9e,
		(byte) 0x7f, (byte) 0xce, (byte) 0x72, (byte) 0xc0, (byte) 0xe9,
		(byte) 0x79, (byte) 0x05, (byte) 0xc5, (byte) 0xfd, (byte) 0x6a,
		(byte) 0x46, (byte) 0xfe, (byte) 0x33, (byte) 0x84, (byte) 0x3f,
		(byte) 0x09, (byte) 0xae, (byte) 0x01, (byte) 0x18, (byte) 0x5a,
		(byte) 0xf6, (byte) 0xc6, (byte) 0xd3, (byte) 0xa1, (byte) 0xe2,
		(byte) 0x90, (byte) 0x83, (byte) 0x79, (byte) 0xee, (byte) 0xa6,
		(byte) 0xd4, (byte) 0xf6, (byte) 0xd1, (byte) 0x86, (byte) 0x91,
		(byte) 0x34, (byte) 0x00, (byte) 0xd3, (byte) 0xe4, (byte) 0x8a,
		(byte) 0xfb, (byte) 0xaa, (byte) 0x6c, (byte) 0xe5, (byte) 0x46,
		(byte) 0xa7, (byte) 0x00, (byte) 0x9e, (byte) 0xd8, (byte) 0x81,
		(byte) 0xbc, (byte) 0xd1, (byte) 0xb5, (byte) 0x60, (byte) 0xd5,
		(byte) 0x91, (byte) 0x13, (byte) 0x06, (byte) 0x68, (byte) 0x21,
		(byte) 0x8f, (byte) 0x7d, (byte) 0xc2, (byte) 0x3e, (byte) 0xd2,
		(byte) 0x75, (byte) 0x0f, (byte) 0x97, (byte) 0x64, (byte) 0xb1,
		(byte) 0xdb, (byte) 0x74, (byte) 0x6e, (byte) 0x91, (byte) 0x6b,
		(byte) 0xa7, (byte) 0x7d, (byte) 0xef, (byte) 0x8b, (byte) 0x37,
		(byte) 0xb7, (byte) 0x84, (byte) 0x1e, (byte) 0xa7, (byte) 0x26,
		(byte) 0x26, (byte) 0xea, (byte) 0xe9, (byte) 0xb7, (byte) 0x5e,
		(byte) 0x3f, (byte) 0xdf, (byte) 0xa4, (byte) 0xc5, (byte) 0x45,
		(byte) 0x4e, (byte) 0x34, (byte) 0x33, (byte) 0xe5, (byte) 0x43,
		(byte) 0x46, (byte) 0xc0, (byte) 0x2b, (byte) 0xbd, (byte) 0x85,
		(byte) 0x2f, (byte) 0xca, (byte) 0xf8, (byte) 0x9d, (byte) 0xb4,
		(byte) 0xbc, (byte) 0x67, (byte) 0x92, (byte) 0xd4, (byte) 0x33,
		(byte) 0xfd, (byte) 0xbd, (byte) 0x82, (byte) 0x9d, (byte) 0x62,
		(byte) 0xfc, (byte) 0xbb, (byte) 0xd2, (byte) 0xad, (byte) 0x05,
		(byte) 0xa2, (byte) 0xfc, (byte) 0x2d, (byte) 0xe3, (byte) 0x02,
		(byte) 0xe2, (byte) 0x41, (byte) 0x9b, (byte) 0x1f, (byte) 0xf8,
		(byte) 0x87, (byte) 0x15, (byte) 0x89, (byte) 0xfb, (byte) 0x53,
		(byte) 0x99, (byte) 0xb3, (byte) 0xeb, (byte) 0xdb, (byte) 0x01,
		(byte) 0xaf, (byte) 0x71, (byte) 0xd2, (byte) 0xf2, (byte) 0x73,
		(byte) 0xb7, (byte) 0x82, (byte) 0x30, (byte) 0x25, (byte) 0x04,
		(byte) 0x29, (byte) 0x2b, (byte) 0xb9, (byte) 0x92, (byte) 0x92,
		(byte) 0x35, (byte) 0x97, (byte) 0x0e, (byte) 0xb8, (byte) 0xf2,
		(byte) 0xc6, (byte) 0x2e, (byte) 0xa7, (byte) 0x2d, (byte) 0x0c,
		(byte) 0x09, (byte) 0x5e, (byte) 0x07, (byte) 0x06, (byte) 0x67,
		(byte) 0xa0, (byte) 0xdf, (byte) 0x55, (byte) 0x09, (byte) 0xfc,
		(byte) 0xee, (byte) 0x2b, (byte) 0x13, (byte) 0x1a, (byte) 0x2e,
		(byte) 0x5d, (byte) 0x0a, (byte) 0xbb, (byte) 0x45, (byte) 0x75,
		(byte) 0xf4, (byte) 0xd8, (byte) 0xdc, (byte) 0x2e, (byte) 0x99,
		(byte) 0x2a, (byte) 0x13, (byte) 0xa1, (byte) 0x1e, (byte) 0x99,
		(byte) 0xfd, (byte) 0xdc, (byte) 0xcf, (byte) 0xcc, (byte) 0x3f,
		(byte) 0x42, (byte) 0xf7, (byte) 0x3d, (byte) 0x73, (byte) 0xee,
		(byte) 0xca, (byte) 0x76, (byte) 0xe4, (byte) 0x75, (byte) 0xc4,
		(byte) 0x21, (byte) 0xd4, (byte) 0x14, (byte) 0x2e, (byte) 0x22,
		(byte) 0x9c, (byte) 0xce, (byte) 0x10, (byte) 0xaf, (byte) 0xa6,
		(byte) 0x25, (byte) 0xa0, (byte) 0x01, (byte) 0xb1, (byte) 0x82,
		(byte) 0xba, (byte) 0x4c, (byte) 0xb2, (byte) 0x66, (byte) 0x89,
		(byte) 0x89, (byte) 0x6b, (byte) 0x06, (byte) 0x15, (byte) 0xba,
		(byte) 0x64, (byte) 0xa3, (byte) 0x73, (byte) 0x88, (byte) 0x34,
		(byte) 0x99, (byte) 0x3e, (byte) 0x75, (byte) 0x24, (byte) 0xf4,
		(byte) 0xba, (byte) 0xb0, (byte) 0x22, (byte) 0x8f, (byte) 0xc3,
		(byte) 0x44, (byte) 0x74, (byte) 0x0b, (byte) 0x52, (byte) 0x96,
		(byte) 0xc6, (byte) 0x97, (byte) 0x8b, (byte) 0xf2, (byte) 0xe3,
		(byte) 0xc1, (byte) 0xaf, (byte) 0x53, (byte) 0x03, (byte) 0x51,
		(byte) 0xa7, (byte) 0x0d, (byte) 0x42, (byte) 0x6a, (byte) 0x20,
		(byte) 0x03, (byte) 0x31, (byte) 0xb4, (byte) 0xc9, (byte) 0xaa,
		(byte) 0x9e, (byte) 0xda, (byte) 0x6f, (byte) 0x7b, (byte) 0xb8,
		(byte) 0x6d, (byte) 0x54, (byte) 0x57, (byte) 0xa8, (byte) 0xed,
		(byte) 0x51, (byte) 0xa4, (byte) 0x23, (byte) 0x05, (byte) 0x0b,
		(byte) 0xb3, (byte) 0x90, (byte) 0x42, (byte) 0x38, (byte) 0xa8,
		(byte) 0xbc, (byte) 0xd5, (byte) 0x2f, (byte) 0x87, (byte) 0x82,
		(byte) 0x5b, (byte) 0xff, (byte) 0xdb, (byte) 0xba, (byte) 0x41,
		(byte) 0x18, (byte) 0xe0, (byte) 0x4a, (byte) 0x07, (byte) 0x04,
		(byte) 0xe1, (byte) 0x3c, (byte) 0xd5, (byte) 0xbf, (byte) 0x3e,
		(byte) 0x40, (byte) 0x7a, (byte) 0x33, (byte) 0x2a, (byte) 0x61,
		(byte) 0x33, (byte) 0xa5, (byte) 0xb4, (byte) 0x06, (byte) 0x96,
		(byte) 0xc0, (byte) 0xaa, (byte) 0xdb, (byte) 0x79, (byte) 0x46,
		(byte) 0xe7, (byte) 0xe5, (byte) 0x6d, (byte) 0xae, (byte) 0x16,
		(byte) 0x6d, (byte) 0xa9, (byte) 0x4a, (byte) 0x39, (byte) 0x0e,
		(byte) 0x5b, (byte) 0x99, (byte) 0x35, (byte) 0x42, (byte) 0xc3,
		(byte) 0xac, (byte) 0xc1, (byte) 0x1b, (byte) 0x6c, (byte) 0x3b,
		(byte) 0xdc, (byte) 0x74, (byte) 0x3b, (byte) 0x52, (byte) 0x5d,
		(byte) 0x74, (byte) 0x30, (byte) 0x77, (byte) 0x3f, (byte) 0x95,
		(byte) 0xf3, (byte) 0x92, (byte) 0xfb, (byte) 0xf6, (byte) 0xd7,
		(byte) 0x94, (byte) 0x49, (byte) 0x63, (byte) 0x56, (byte) 0xd4,
		(byte) 0x8c, (byte) 0x7d, (byte) 0x56, (byte) 0x84, (byte) 0xca,
		(byte) 0x54, (byte) 0x77, (byte) 0x29, (byte) 0x3b, (byte) 0xa7,
		(byte) 0x60 };
//==================== Cyclic Seed File Ends====================================
	
	
	
	
	private final short FILE_SIZE = (short)528;
	private final byte RECORD_SIZE = (byte)16;
	private static short CyclicRecordReadPointer = 16; // point one record ahead
	private static short CyclicRecordWritePointer = 0; // point one record behind
	private static byte[] fixedKey = {(byte) 0x5b, (byte) 0xff, (byte) 0xdb, (byte) 0xba, (byte) 0x41,
		(byte) 0x18, (byte) 0xe0, (byte) 0x4a,(byte) 0x74, (byte) 0x30, (byte) 0x77, (byte) 0x3f, (byte) 0x95,
		(byte) 0xf3, (byte) 0x92, (byte) 0xfb };
	
	
	
	/*Java Applet Specific Constants
	 * */
	// CLA byte of MyCounter Applet
	final static byte CLA = (byte)0xB0;
	
	// INS bytes of MyCounter Applet
	final static byte GETRND = (byte)0x40;
	
	
	// APPlets ERR Status Words	
	final static short SW_CLASSNOTSUPPORTED = 0x6320;
	final static short SW_ERROR_INS = 0x6300;
	// A 32 byte buffer for output storage of SHA1 (160 bits)
	byte[] Buffer = JCSystem.makeTransientByteArray((short)16, JCSystem.CLEAR_ON_DESELECT);
	Cipher cipherAES;
	AESKey transKey = (AESKey)KeyBuilder.buildKey(KeyBuilder.TYPE_AES_TRANSIENT_DESELECT, 
			KeyBuilder.LENGTH_AES_128, false);
	

	
	
	

	//=============== Utility Methods==============================

		
		
		void Xor(byte[] inputBuffer, byte condition){
			if(condition == 1){				
				// XORing with previous value to create new seed
				for(short i=0, j=RECORD_SIZE; i<RECORD_SIZE; i++, j++){
					
					inputBuffer[i] = (byte)(inputBuffer[(i)] ^ inputBuffer[j]);
						
					}
			}else{
				for(short i=0; i<16; i++){
					inputBuffer[i]= (byte)(inputBuffer[i] ^ inputBuffer[(byte)(31-i)]);
				}
			}
			
		}
		
		/*
		 * CyclicSeedFile record reading and writing is performed in Two Functions
		 * 1) CyclicSeedFileRead
		 * 2) CyclicSeedFileWrite
		 * */
		private void CyclicSeedFileRead(byte[] readBuffer){
			if(CyclicRecordReadPointer >= FILE_SIZE)
				CyclicRecordReadPointer = 0;
			
			Util.arrayCopyNonAtomic(CyclicSeedFile, CyclicRecordReadPointer, readBuffer, (short)0,(short)(RECORD_SIZE));

			// incrementing for next cycle
			CyclicRecordReadPointer = (short)((short)CyclicRecordReadPointer + RECORD_SIZE);
		}
		
		private void CyclicSeedFileWrite(byte[] writeBuffer){		
			if(CyclicRecordWritePointer >= FILE_SIZE)
				CyclicRecordWritePointer = 0;
			
			Util.arrayCopyNonAtomic(CyclicSeedFile, CyclicRecordWritePointer, writeBuffer, (short)0, RECORD_SIZE);
			
				Xor(writeBuffer, (byte)0);
				
			
			Util.arrayCopyNonAtomic(writeBuffer,(byte)0, CyclicSeedFile, CyclicRecordWritePointer, RECORD_SIZE);
			
				// incrementing for next cycle
			CyclicRecordWritePointer = (short)(CyclicRecordWritePointer + RECORD_SIZE);
		}
			
		
		
		//==================== Utility Mehthods End====================================
	
	
	
	private PrngAES() {
		cipherAES = Cipher.getInstance(Cipher.ALG_AES_BLOCK_128_CBC_NOPAD, false);
		register();
	}

	public static void install(byte bArray[], short bOffset, byte bLength)
			throws ISOException {
		new PrngAES();
	}


	public void process(APDU apdu) throws ISOException {
		byte[] apduBuffer = apdu.getBuffer();
		
		// Return if the APDU is selection APDU	
		if(selectingApplet()){
				return;
			}
			
		// For Command APDUs first Check its CLA value
		if(apduBuffer[ISO7816.OFFSET_CLA]!= CLA){
			ISOException.throwIt(SW_CLASSNOTSUPPORTED);
		}else if(apduBuffer[ISO7816.OFFSET_INS] != GETRND){//For Command APDUs second Check its INS value
			ISOException.throwIt(SW_ERROR_INS);
		}
			/***************************** Actual Algorithm Implementation *****************************************************/
				
				
				
				CyclicSeedFileRead(apduBuffer);
				//KeySeedExtractor(apduBuffer,(short)0);
				transKey.setKey(fixedKey, (short)0);
					cipherAES.init(transKey, Cipher.MODE_ENCRYPT);
					cipherAES.doFinal(apduBuffer,(short) 0,RECORD_SIZE, apduBuffer,RECORD_SIZE);
				Xor(apduBuffer,(byte)1);
				CyclicSeedFileWrite(apduBuffer);
				//KeySeedExtractor(apduBuffer, (short)16);				
					//CipherDES.init(transKey, Cipher.MODE_ENCRYPT);
					cipherAES.doFinal(apduBuffer, (short)16, RECORD_SIZE, apduBuffer, (short)0);
													
				apdu.setOutgoingAndSend((short)0, (short)16);

	}

}
